/**
 * crc16.c
 *
 * CRC 16 implementation.
 *
 * Author: Christian Jung <chris@tapage.de>
 * Date  : 2009-06-06
 */

#include "crc16.h"

/* CRC 16 Table algorithm */
uint16_t crc16_table[256];


/**
 * Initialize table of CRC algorithm for the given CRC polynom.
 */
void crc16_init_table( const uint16_t polynom ) {
	uint16_t r;
	int i, j;

	for ( i = 0; i < 256; i++ ) {
		r = (uint16_t)(i << 8);
		for ( j = 0; j < 8; j++ ) {
			if ( r & 0x8000 ) {
				r = (r << 1) ^ polynom;
			} else {
				r <<= 1;
			}
		}	
		crc16_table[i] = r;
	}
}


/**
 * Initialize CRC with given value 'init'.
 */
uint16_t crc16_init( const uint16_t init ) {
	uint16_t crc = crc16_byte( 0x0000, (uint8_t)(init >> 8) );
	return crc16_byte( crc, (uint8_t)(init & 0xff) );
}


/**
 * Compute the CRC of the given buffer with the initial CRC-value 'init'.
 */
uint16_t crc16( const uint16_t init, uint8_t const *buffer, size_t length ) {
	uint16_t crc = crc16_init( init );
	while ( length-- ) crc = crc16_byte( crc, *buffer++ );
	return crc;
}
