/**
 * lcp100.h
 *
 * Author: Christian Jung <chris@tapage.de>
 * Date  : 2009-06-06
 */

#ifndef _LCP100_H
#define _LCP100_H

#include <string.h>
#include <termios.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdlib.h>
#include "frame.h"

/* enable workaround for "clear display" bug: */
#define LCP100_WORKAROUND_CLEAR

/* commands */
#define LCP100_CMD_CLEAR       0x65 /* clear display                   */
#define LCP100_CMD_ST7637_CMD  0x69 /* send command directly to ST7637 */
#define LCP100_CMD_LOAD_PAGE   0x6c /* download page from memory       */
#define LCP100_CMD_ST7637_DATA 0x6f /* send data directly to ST7637    */
#define LCP100_CMD_SAVE_PAGE   0x70 /* upload page to memory           */
#define LCP100_CMD_SHOW_PAGE   0x73 /* display page                    */
#define LCP100_CMD_TEST        0x74 /* show test image                 */
#define LCP100_CMD_VERSION     0x76 /* get version of firmware         */
#define LCP100_CMD_BACKLIGHT   0x7a /* control backlight               */

/* protocol stuff */
#define LCP100_MAX_PACKET_SIZE 0xffff
#define LCP100_CRC16_POLYNOM   0x8005
#define LCP100_PAGE_SIZE       0x200
#define LCP100_PAGES_PER_IMAGE 64
#define LCP100_NUMBER_PAGES    4096
#define LCP100_ACK             0x06
#define LCP100_NAK             0x15
#define LCP100_BACKLIGHT_ON    0x01
#define LCP100_BACKLIGHT_OFF   0x00

int lcp100_open( const char *device );
int lcp100_init( const int fd );
int lcp100_version( const int fd, int *major, int *minor );
int lcp100_clear( const int fd);
int lcp100_test_image( const int fd );
int lcp100_backlight( const int fd, const int status );
int lcp100_page_upload( const int fd, uint16_t index, const uint8_t *page, size_t length );
int lcp100_page_show( const int fd, uint16_t index, uint16_t length );
int lcp100_st7637_cmd( const int fd, const uint8_t command ) ;
int lcp100_st7637_data( const int fd, const uint8_t data );

#endif /* _LCP100_H */
